"""
This function simulates the random forest based neural network
"""

import os
os.environ["THEANO_FLAGS"] = "mode=FAST_RUN,device=cpu,floatX=float32"
import numpy
from theano import tensor as T
import theano
import os
import six.moves.cPickle as pickle
import gzip
from collections import OrderedDict
from theano import config
import timeit
import scipy.io
from lasagne.updates import adam, adadelta, momentum
import cPickle
from sklearn.utils import resample
import argparse

def get_minibatches_idx(n, minibatch_size, rng, shuffle=False):
    """
    Used to shuffle the dataset at each iteration.
    """

    idx_list = numpy.arange(n, dtype="int32")

    if shuffle:
        # numpy.random.shuffle(idx_list)
        rng.shuffle(idx_list)

    minibatches = []
    minibatch_start = 0
    for i in range(n // minibatch_size):
        minibatches.append(idx_list[minibatch_start:
                                    minibatch_start + minibatch_size])
        minibatch_start += minibatch_size

    if (minibatch_start != n):
        # Make a minibatch out of what is left
        minibatches.append(idx_list[minibatch_start:])

    return zip(range(len(minibatches)), minibatches)

class root_node(object):
    def __init__(self, rng, input, n_in, n_out, n_total_feas, fea_ind=None, W=None, b=None,
                 activation=T.nnet.relu):
        """
        Typical hidden layer of a MLP: units are fully-connected and have
        sigmoidal activation function. Weight matrix W is of shape (n_in,n_out)
        and the bias vector b is of shape (n_out,).

        NOTE : The nonlinearity used here is tanh

        Hidden unit activation is given by: tanh(dot(input,W) + b)

        :type rng: numpy.random.RandomState
        :param rng: a random number generator used to initialize weights

        :type input: theano.tensor.dmatrix
        :param input: a symbolic tensor of shape (n_examples, n_in)

        :type n_in: int
        :param n_in: dimensionality of input

        :type n_out: int
        :param n_out: number of hidden units

        :type activation: theano.Op or function
        :param activation: Non linearity to be applied in the hidden
                           layer
        """

        # end-snippet-1

        # `W` is initialized with `W_values` which is uniformely sampled
        # from sqrt(-6./(n_in+n_hidden)) and sqrt(6./(n_in+n_hidden))
        # for tanh activation function
        # the output of uniform if converted using asarray to dtype
        # theano.config.floatX so that the code is runable on GPU
        # Note : optimal initialization of weights is dependent on the
        #        activation function used (among other things).
        #        For example, results presented in [Xavier10] suggest that you
        #        should use 4 times larger initial weights for sigmoid
        #        compared to tanh
        #        We have no info for other function, so we use the same as
        #        tanh.
        if W is None:
            # W_values = numpy.asarray(
            #     rng.uniform(
            #         low=-numpy.sqrt(6. / (n_in + n_out)),
            #         high=numpy.sqrt(6. / (n_in + n_out)),
            #         size=(n_in, n_out)
            #     ),
            #     dtype=theano.config.floatX
            # )
            # W_values = 0.01 * numpy.random.randn(n_in, n_out)
            W_values = 0.01 * rng.randn(n_in, n_out)
            if activation == theano.tensor.nnet.sigmoid:
                W_values *= 4

            W = theano.shared(value=W_values, name='W', borrow=True)

        if b is None:
            b_values = numpy.zeros((n_out,), dtype=theano.config.floatX)
            b = theano.shared(value=b_values, name='b', borrow=True)
        if fea_ind is None:
            rand_ind = rng.permutation(n_total_feas)[0:n_in].astype('int32')
            fea_ind = theano.shared(value=rand_ind, name='fea_ind', borrow=True)
        self.fea_ind = fea_ind

        self.W = W
        self.b = b

        lin_output = T.dot(input[:,self.fea_ind], self.W) + self.b
        self.output = (
            lin_output if activation is None
            else activation(lin_output, alpha=0.2)
        )

        self.out_ind = T.argmax(self.output, axis=1)
        self.one_output = T.max(self.output, axis=1)

        # parameters of the model
        self.params = [self.W, self.b]

        self.input = input  #TODO: to be optimized, store original inputs or subinputs?


class hidden_layer(object):
    def __init__(self, rng, input, input1, ind, n_in, n_out, n_total_feas, fea_ind=None, W=None, b=None,
                 activation=T.nnet.relu):
        """
        Typical hidden layer of a MLP: units are fully-connected and have
        sigmoidal activation function. Weight matrix W is of shape (n_in,n_out)
        and the bias vector b is of shape (n_out,).

        NOTE : The nonlinearity used here is tanh

        Hidden unit activation is given by: tanh(dot(input,W) + b)

        :type rng: numpy.random.RandomState
        :param rng: a random number generator used to initialize weights

        :type input: theano.tensor.dmatrix
        :param input: a symbolic tensor of shape (n_examples, n_in)

        :type n_in: int
        :param n_in: dimensionality of input

        :type n_out: int
        :param n_out: number of hidden units

        :type activation: theano.Op or function
        :param activation: Non linearity to be applied in the hidden
                           layer
        """
        if W is None:
            # W_values = numpy.asarray(
            #     rng.uniform(
            #         low=-numpy.sqrt(6. / (n_in + n_out)),
            #         high=numpy.sqrt(6. / (n_in + n_out)),
            #         size=(n_out, n_in)
            #     ),
            #     dtype=theano.config.floatX
            # )
            # W_values = 0.01 * numpy.random.randn(n_out, n_in)
            W_values = 0.01 * rng.randn(n_out, n_in)
            if activation == theano.tensor.nnet.sigmoid:
                W_values *= 4

            W = theano.shared(value=W_values, name='W', borrow=True)

        if b is None:
            b_values = numpy.zeros((n_out,), dtype=theano.config.floatX)
            b = theano.shared(value=b_values, name='b', borrow=True)

        self.W = W
        self.b = b
        self.input = input
        self.input1 = input1
        self.ind = ind

        if fea_ind is None:
            fea_ind_value = rng.permutation(n_total_feas)[0:(n_in - 1)]
            for i in range(1,n_out/2):
                fea_ind_value = numpy.concatenate([fea_ind_value,rng.permutation(n_total_feas)[0:(n_in-1)]])
            fea_ind_value = fea_ind_value.astype('int32')
            fea_ind = theano.shared(value=fea_ind_value, name='fea_ind', borrow=True)
        self.fea_ind = fea_ind

        valid_input = T.concatenate([T.reshape(input[:,self.fea_ind], (input.shape[0],n_out/2,n_in-1))[T.arange(input.shape[0]),self.ind,:], self.input1], axis=1)

        output1 = activation(T.sum(valid_input * self.W[2*self.ind,:], axis=1) + self.b[2*self.ind], alpha=0.2)
        output2 = activation(T.sum(valid_input * self.W[2*self.ind+1,:], axis=1) + self.b[2*self.ind+1], alpha=0.2)


        # output1 = activation(T.sum(self.input * self.W[2*self.ind,:], axis=1) + self.b[2*self.ind], alpha=0.2)
        # output2 = activation(T.sum(self.input * self.W[2*self.ind+1,:], axis=1) + self.b[2*self.ind+1], alpha=0.2)

        self.out_ind = 2*self.ind + (output2 < output1)
        self.output = T.max((output1,output2), axis=0) # TODO: check this

        # parameters of the model
        self.params = [self.W, self.b]

class hidden_layer_last(object):
    def __init__(self, rng, input, input1, ind, n_in, n_out, n_total_feas, fea_ind=None, W=None, b=None,
                 activation=T.nnet.relu):
        """
        Typical hidden layer of a MLP: units are fully-connected and have
        sigmoidal activation function. Weight matrix W is of shape (n_in,n_out)
        and the bias vector b is of shape (n_out,).

        NOTE : The nonlinearity used here is tanh

        Hidden unit activation is given by: tanh(dot(input,W) + b)

        :type rng: numpy.random.RandomState
        :param rng: a random number generator used to initialize weights

        :type input: theano.tensor.dmatrix
        :param input: a symbolic tensor of shape (n_examples, n_in)

        :type n_in: int
        :param n_in: dimensionality of input

        :type n_out: int
        :param n_out: number of hidden units

        :type activation: theano.Op or function
        :param activation: Non linearity to be applied in the hidden
                           layer
        """
        if W is None:
            # W_values = numpy.asarray(
            #     rng.uniform(
            #         low=-numpy.sqrt(6. / (n_in + n_out)),
            #         high=numpy.sqrt(6. / (n_in + n_out)),
            #         size=(n_out, n_in)
            #     ),
            #     dtype=theano.config.floatX
            # )
            # W_values= 0.01 * numpy.random.randn(n_out, n_in)
            W_values = 0.01 * rng.randn(n_out, n_in)
            if activation == theano.tensor.nnet.sigmoid:
                W_values *= 4

            W = theano.shared(value=W_values, name='W', borrow=True)

        if b is None:
            b_values = numpy.zeros((n_out,), dtype=theano.config.floatX)
            b = theano.shared(value=b_values, name='b', borrow=True)

        self.W = W
        self.b = b
        self.input = input
        self.input1 = input1
        self.ind = ind

        if fea_ind is None:
            fea_ind_value = rng.permutation(n_total_feas)[0:(n_in - 1)]
            for i in range(1,n_out/2):
                fea_ind_value = numpy.concatenate([fea_ind_value,rng.permutation(n_total_feas)[0:(n_in-1)]])
            fea_ind_value = fea_ind_value.astype('int32')
            fea_ind = theano.shared(value=fea_ind_value, name='fea_ind', borrow=True)
        self.fea_ind = fea_ind

        valid_input = T.concatenate([T.reshape(input[:,self.fea_ind], (input.shape[0],n_out/2,n_in-1))[T.arange(input.shape[0]),self.ind,:], self.input1], axis=1)

        output1 = activation(T.sum(valid_input * self.W[2*self.ind,:], axis=1) + self.b[2*self.ind], alpha=0.2)
        output2 = activation(T.sum(valid_input * self.W[2*self.ind+1,:], axis=1) + self.b[2*self.ind+1], alpha=0.2)


        # output1 = activation(T.sum(self.input * self.W[2*self.ind,:], axis=1) + self.b[2*self.ind], alpha=0.2)
        # output2 = activation(T.sum(self.input * self.W[2*self.ind+1,:], axis=1) + self.b[2*self.ind+1], alpha=0.2)

        self.out_ind = 2*self.ind + (output2 < output1)
        output = T.zeros((self.input.shape[0], n_out))
        output = T.set_subtensor(output[T.arange(self.input.shape[0]), 2*self.ind], output1)
        output = T.set_subtensor(output[T.arange(self.input.shape[0]), 2*self.ind+1], output2)

        self.output = output

        # parameters of the model
        self.params = [self.W, self.b]

class prediction_layer(object):
    def __init__(self, input, ind, n_in, n_class):
        """ Initialize the parameters of the type_three_node

                :param input:
                :param n_in:
                :param n_out:
                """
        # initialize with 0 the weights W as a matrix of shape (n_in, n_out)

        ## Attention: only one path is activated currently, check if need two path for the last layer

        self.W = theano.shared(
            value=numpy.zeros(
                (n_in, n_class),
                dtype=theano.config.floatX
            ),
            name='W',
            borrow=True
        )
        # initialize the biases b as a vector of n_out 0s
        self.b = theano.shared(
            value=numpy.zeros(
                (n_class,),
                dtype=theano.config.floatX
            ),
            name='b',
            borrow=True
        )

        # prediction
        self.p_y_given_x = T.nnet.softmax(input.dimshuffle(0,'x') * self.W[ind,:] + self.b)

        # self.p_y_given_x = T.nnet.softmax(T.dot(input, self.W) + self.b)

        self.y_pred = T.argmax(self.p_y_given_x, axis=1)

        self.params = [self.W, self.b]

        self.input = input
        self.ind = ind

class type_three_node(object):
    def __init__(self, input, n_in, n_out, rng):
        """ Initialize the parameters of the type_three_node

        :param input:
        :param n_in:
        :param n_out:
        """
        # initialize with 0 the weights W as a matrix of shape (n_in, n_out)
        # self.W = theano.shared(
        #     value=numpy.random.randn(
        #         n_in, n_out
        #     ),
        #     name='W',
        #     borrow=True
        # )
        # self.W = theano.shared(
        #     value=0.01 * numpy.random.randn(
        #         n_in, n_out
        #     ),
        #     name='W',
        #     borrow=True
        # )
        self.W = theano.shared(
            value=0.01 * rng.randn(
                n_in, n_out
            ),
            name='W',
            borrow=True
        )
        # initialize the biases b as a vector of n_out 0s
        self.b = theano.shared(
            value=numpy.zeros(
                (n_out,),
                dtype=theano.config.floatX
            ),
            name='b',
            borrow=True
        )

        # prediction
        self.p_y_given_x = T.nnet.softmax(T.dot(input, self.W) + self.b)

        self.y_pred = T.argmax(self.p_y_given_x, axis=1)

        self.params = [self.W, self.b]

        self.input = input

class NNDT(object):
    """Decision tree like neural network

    The structure of the network looks like a binary tree, where each
    node need to decide which direct to take, which simulates the decision
    tree. Each input will activate one and only one path so that it is
    efficient to perform forward and backward propagation. The final layer
    aggregrates all the output using softmax function (or linear classifier).

    """
    def __init__(self, rng, input, n_in, n_class, n_level, n_features):
        """ Initialize the parameters for the decision like neural network

        :param rng:
        :param input:
        :param n_in:
        :param n_class:
        :param n_level:
        :param n_out:
        """
        # root node

        self.root_node = root_node(
            rng=rng,
            input=input,
            n_in=n_in,
            n_out=2,
            n_total_feas=n_features,
            activation=T.nnet.relu
        )

        # layer 1
        self.layer1 = hidden_layer(
            rng=rng,
            input=input,
            input1=self.root_node.one_output.dimshuffle(0, 'x'),
            ind=self.root_node.out_ind,
            n_in=n_in+1,
            n_total_feas=n_features,
            n_out=2**2
        )

        # layer 2
        self.layer2 = hidden_layer_last(
            rng=rng,
            input=input,
            input1=self.layer1.output.dimshuffle(0,'x'),
            ind=self.layer1.out_ind,
            n_in=n_in+1,
            n_total_feas=n_features,
            n_out=2**3
        )

        # # layer 3
        # self.layer3 = hidden_layer(
        #     rng=rng,
        #     input=input,
        #     input1=self.layer2.output.dimshuffle(0, 'x'),
        #     ind=self.layer2.out_ind,
        #     n_in=n_in+1,
        #     n_total_feas=n_features,
        #     n_out=2**4
        # )
        #
        # # layer 4
        # self.layer4 = hidden_layer_last(
        #     rng=rng,
        #     input=input,
        #     input1=self.layer3.output.dimshuffle(0, 'x'),
        #     ind=self.layer3.out_ind,
        #     n_in=n_in+1,
        #     n_total_feas=n_features,
        #     n_out=2**5
        # )


        # predict layer
        self.predict_layer = type_three_node(
            input=self.layer2.output,
            n_in=2**n_level,
            n_out=n_class,
            rng=rng
        )

        self.input = input

        self.p_y_given_x = self.predict_layer.p_y_given_x
        self.y_pred = self.predict_layer.y_pred

        # parameters
        self.params = (
            self.root_node.params
            + self.layer1.params
            + self.layer2.params
            # + self.layer3.params
            # + self.layer4.params
            + self.predict_layer.params
        )

        self.L2_sqr = (
            (self.root_node.W ** 2).sum()
            + (self.layer1.W ** 2).sum()
            + (self.layer2.W ** 2).sum()
            # + (self.layer3.W ** 2).sum()
            # + (self.layer4.W ** 2).sum()
            + (self.predict_layer.W ** 2).sum()
        )

    def negative_log_likelihood(self, y):
        return -T.mean(T.log(self.p_y_given_x)[T.arange(y.shape[0]), y])

    def errors(self, y):
        """Return a float representing the number of errors in the minibatch
        over the total number of examples of the minibatch ; zero one
        loss over the size of the minibatch

        :type y: theano.tensor.TensorType
        :param y: corresponds to a vector that gives for each example the
                  correct label
        """

        # check if y has same dimension of y_pred
        if y.ndim != self.y_pred.ndim:
            raise TypeError(
                'y should have the same shape as self.y_pred',
                ('y', y.type, 'y_pred', self.y_pred.type)
            )
        # check if y is of the correct datatype
        if y.dtype.startswith('int'):
            # the T.neq operator returns a vector of 0s and 1s, where 1
            # represents a mistake in prediction
            return T.mean(T.neq(self.y_pred, y)), self.y_pred, self.p_y_given_x
        else:
            raise NotImplementedError()

def load_mat_data(dataset):
    # mat = scipy.io.loadmat(dataset)
    # trainX = theano.shared(numpy.asarray(mat['trainX'].astype(numpy.float32,copy=False),
    #                                      dtype=theano.config.floatX),
    #                        borrow=True)
    # trainY = theano.shared(numpy.asarray(mat['trainY'].ravel()-1,
    #                                      dtype='int32'),
    #                        borrow=True)
    # testX = theano.shared(numpy.asarray(mat['testX'].astype(numpy.float32,copy=False),
    #                                     dtype=theano.config.floatX),
    #                        borrow=True)
    # testY = theano.shared(numpy.asarray(mat['testY'].ravel()-1,
    #                                     dtype='int32'),
    #                        borrow=True)
    # return trainX, trainY, testX, testY
    mat = cPickle.load(open(dataset, 'rb'))
    # mat = scipy.io.loadmat(dataset)
    # X = numpy.concatenate([mat['trainX'], mat['testX']], axis=0)
    # X -= numpy.mean(X, axis = 0)
    # X /= numpy.std(X, axis=0)
    # X = X.astype(numpy.float32)
    # num_train = mat['trainX'].shape[0]
    # num_test = mat['testX'].shape[0]
    # mat['trainX'] = X[0:num_train,:]
    # mat['testX'] = X[num_train:num_train+num_test,:]
    mean_value = numpy.mean(mat['trainX'], axis=0)
    std_value = numpy.std(mat['trainX'], axis=0)
    mat['trainX'] -= mean_value
    mat['testX'] -= mean_value
    mat['trainX'] /= std_value
    mat['testX'] /= std_value


    trainX = theano.shared(numpy.asarray(mat['trainX'].astype(numpy.float32,copy=False),
                                         dtype=theano.config.floatX),
                           borrow=True)
    trainY = theano.shared(numpy.asarray(mat['trainY'].ravel()-1,
                                         dtype='int32'),
                           borrow=True)
    testX = theano.shared(numpy.asarray(mat['testX'].astype(numpy.float32,copy=False),
                                        dtype=theano.config.floatX),
                           borrow=True)
    testY = theano.shared(numpy.asarray(mat['testY'].ravel()-1,
                                        dtype='int32'),
                           borrow=True)
    return trainX, trainY, testX, testY


def test_NNDT(
        rng,
        train_set_x,
        train_set_y,
        test_set_x,
        test_set_y,
        n_features,
        n_train,
        n_test,
        n_class,
        tree_ind,
        n_in,
        learning_rate=0.01,
        L2_reg=0.01,
        n_epochs=1000,
        batch_size=20
):

    """

    :param learning_rate:
    :param L1_ref:
    :param L2_reg:
    :param n_epochs:
    :param dataset:
    :param batch_size:
    :return:
    """

    # Model options
    model_options = locals().copy()
    print("model options", model_options)

    # bootstrap
    bootstrap_sample = resample(numpy.arange(n_train), random_state=rng).astype("int32")

    # load data
    # mat = cPickle.load(open(dataset, 'rb'))
    # train_set_x, train_set_y, test_set_x, test_set_y = load_mat_data(dataset)

    #rng = numpy.random.RandomState(1234)

    ######################
    # BUILD ACTUAL MODEL #
    ######################
    print('... building the model')

    index = T.ivector('idx')
    x = T.matrix('x')
    y = T.ivector('y')

    # construct the NNDT class
    classifier = NNDT(
        rng=rng,
        input=x,
        n_in=n_in,
        n_features=n_features, # mat['trainX'].shape[1],
        n_class=n_class, # len(numpy.unique(mat['trainY'])),
        n_level=3
    )

    cost = (
        classifier.negative_log_likelihood(y)
        + L2_reg * classifier.L2_sqr
    )

    validate_model = theano.function(
        inputs=[index],
        outputs=classifier.errors(y),
        givens={
            x: train_set_x[index],  # use validation set to replace train_set_x
            y: train_set_y[index]   # use validation set to replace train_set_y
        }
    )

    test_model = theano.function(
        inputs=[index],
        outputs=classifier.errors(y),
        givens={
            x: test_set_x[index],
            y: test_set_y[index]
        }
    )

    # get_ind = theano.function(outputs=classifier.layer1.fea_ind)

    # compute the gradient of cost with respect to theta (sorted in params)
    # the resulting gradients will be stored in a list gparams
    gparams = [T.grad(cost, param) for param in classifier.params]

    # specify how to update the parameters of the model as a list of
    # (variable, update expression) pairs

    # given two lists of the same length, A = [a1, a2, a3, a4] and
    # B = [b1, b2, b3, b4], zip generates a list C of same size, where each
    # element is a pair formed from the two lists :
    #    C = [(a1, b1), (a2, b2), (a3, b3), (a4, b4)]

    l_r = T.scalar('l_r', dtype=theano.config.floatX)

    # updates = [
    #     (param, param - l_r * gparam)
    #     for param, gparam in zip(classifier.params, gparams)
    #     ]

    updates = momentum(loss_or_grads=cost, params=classifier.params, learning_rate=l_r)

    # updates = adadelta(loss_or_grads=cost, params=classifier.params, learning_rate=10.0, rho=0.95, epsilon=1e-6)

    train_model = theano.function(
        inputs=[index, l_r],
        outputs=cost,
        updates=updates,
        givens={
            x: train_set_x[index],
            y: train_set_y[index]
        }
    )

    # get_value = theano.function(
    #     inputs=[index],
    #     outputs=[classifier.type_one_node.output,classifier.last_layer.input],
    #     givens={
    #         x: train_set_x[index]
    #     }
    # )
    # get_value_indicator = theano.function(
    #     inputs=[index],
    #     outputs=[classifier.first_indicator,classifier.second_indicator,classifier.last_indicator],
    #     givens={
    #         x: train_set_x[index]
    #     }
    # )

    # get_value_indicator = theano.function(
    #     inputs=[index],
    #     outputs=[classifier.last_indicator],
    #     givens={
    #         x: train_set_x[index]
    #     }
    # )

    ###############
    # TRAIN MODEL #
    ###############
    print('... training')

    start_time = timeit.default_timer()

    epoch = 0
    done_looping = False

    while (epoch < n_epochs) and (not done_looping):
        epoch += 1

        # Get new shuffled index for the training set.
        kf = get_minibatches_idx(n_train, batch_size, rng=rng, shuffle=True)

        avg_cost = 0

        for minibatch_index, train_index in kf:
            minibatch_avg_cost = train_model(bootstrap_sample[train_index], learning_rate)
            # first_value, value = get_value(train_index)
            # first_indicator, second_indicator, indicator = get_value_indicator(train_index)
            # print(value)
            # indicator = get_value_indicator(train_index)
            # print indicator
            avg_cost += minibatch_avg_cost * len(train_index)

        print(avg_cost / n_train)

        # change learning rate
        if epoch % 5 == 0:
            learning_rate = learning_rate * 0.95
        #learning_rate = learning_rate * 0.95

    # Note: change 0.25 for different dataset. Adding this is to avoid bad decision tree
    # like neural network
    train_error, train_pred, train_ppred = validate_model(numpy.arange(n_train, dtype='int32'))
    print('\t%f' %(train_error,))

    if train_error < 0.5:
        test_score, test_pred, test_ppred = test_model(numpy.arange(n_test, dtype='int32'))
        outf = open('./res/%d.p' %(
            tree_ind,), 'wb')
        cPickle.dump([test_pred, test_ppred], outf)


def test_NNRF(learning_rate=0.01,
              L2_reg=0.01,
              n_epochs=1000,
              dataset='dataset',
              batch_size=20,
              n_in=16,
              num_tree=100):
    """
    :param learning_rate:
    :param L2_reg:
    :param n_epochs:
    :param dataset:
    :param batch_size:
    :param n_in: number of features to sample for each tree
    :param num_tree:
    :return:
    """
    # Model options
    model_options = locals().copy()
    print("model options", model_options)

    # load data
    mat = cPickle.load(open(dataset, 'rb'))
    # mat = scipy.io.loadmat(dataset)
    train_set_x, train_set_y, test_set_x, test_set_y = load_mat_data(dataset)

    rng = numpy.random.RandomState(1496)

    for i in xrange(num_tree):
        test_NNDT(
            rng=rng,
            train_set_x=train_set_x,
            train_set_y=train_set_y,
            test_set_x=test_set_x,
            test_set_y=test_set_y,
            n_features=mat['trainX'].shape[1],
            n_train=mat['trainX'].shape[0],
            n_test=mat['testX'].shape[0],
            n_class=len(numpy.unique(mat['trainY'])),
            tree_ind=i,
            n_in=n_in,
            learning_rate=learning_rate,
            L2_reg=L2_reg,
            n_epochs=n_epochs,
            batch_size=batch_size
        )


# start testing
if __name__ == '__main__':
    # test_NNDT(
    #     learning_rate=1.8,
    #     L2_reg=0.0001,
    #     n_epochs=400,
    #     dataset='../dataset/usps.p',
    #     batch_size=400
    # )

    # parser = argparse.ArgumentParser()
    # parser.add_argument("learning_rate", help="learning rate", type=float)
    # parser.add_argument("batch_size", help="batch size", type=int)
    # parser.add_argument("num_tree", help="number of trees", type=int)
    # args = parser.parse_args()
    #
    # learning_rate = args.learning_rate
    # batch_size = args.batch_size
    # num_tree = args.num_tree

    learning_rate = 1
    batch_size = 200
    num_tree = 100

    test_NNRF(
        learning_rate=learning_rate,
        L2_reg=0.0005,
        n_epochs=500,
        dataset='../dataset/wine.p',
        batch_size=batch_size,
        n_in=5,
        num_tree=num_tree
    )
    #
    # pred =
    # for i in xrange(num_tree):
    #     cPickle.load(open('../res/' + str(i) + '.p', 'rb'))
